@@ -13,21 +13,37 @@ from PyQt5 import QtCore, QtGui, QtWidgets
 
 import json
 
-from cryspy import GlobalN, file_to_globaln, RhoChi
+from cryspy import GlobalN, file_to_globaln, RhoChi, load_packages
+
 from cryspy import __version__ as cryspy_version
 from cryspy_editor import __version__ as cryspy_editor_version
 
 from cryspy_editor.widgets.w_panel import WPanel, \
     find_tree_item_position, L_GLOBAL_CLS
-from cryspy_editor.widgets.w_methods import WMethods
 from cryspy_editor.widgets.w_baseobj import WBaseObj
-from cryspy_editor.widgets.w_dockarea import WDockArea
 from cryspy_editor.widgets.w_console import WConsole
 from cryspy_editor.widgets.w_function import WFunction
+from cryspy_editor.widgets.w_variables import WVariables
+# from cryspy_editor.widgets.w_notes import WNotes
+from cryspy_editor.widgets.w_packages import WPackages
 
-# import importlib
-# dir_additional_libraries = d_setup['directory_additional_libraries']
+from cryspy_editor.widgets.w_presentation import define_tool_buttons
 
+S_NOTES = """# Cryspy Editor
+
+The program is used for data refinement. Open example file from GitHub page \
+to start the work.
+
+
+## Notes
+
+  - In this window you can write own notes in markdown format. To save it in \
+the file use the context menu.
+  - In tab "Notes" the notes are presented in html format.
+
+"""
+
+load_packages()
 
 def text_from_d_info(d_info: dict = None):
     """Take text from print key of d_info."""
@@ -143,11 +159,11 @@ class CBuilder(QtWidgets.QMainWindow):
         - wfunction
         - wpanel
         - wconsole
+        - wnotes
         - object
         - wbaseobj
-        - wdockarea
         - wcb_for_stackedwidget
-        - wstackwidget
+        - wstackedwidget
     """
 
     def __init__(self, f_dir_data=os.path.dirname(__file__)):
@@ -156,6 +172,8 @@ class CBuilder(QtWidgets.QMainWindow):
         self.f_dir_prog = os.path.dirname(__file__)
         self.index_curent_item = None
 
+        self.dsetup = self.get_dsetup()
+
         self.def_actions()
         self.init_widget()
 
@@ -171,7 +189,7 @@ class CBuilder(QtWidgets.QMainWindow):
         screen = QtWidgets.QDesktopWidget().screenGeometry()
         self.mythread.window.resize(screen.width() * 0.5,
                                     screen.height() * 0.2)
-        f_calc = os.path.join(self.f_dir_prog, "f_icon", "calc.gif")
+        f_calc = os.path.join(self.f_dir_prog, "f_icon", "pm.gif")
         q_label = QtWidgets.QLabel()
         gif = QtGui.QMovie(f_calc)
         q_label.setMovie(gif)
@@ -179,12 +197,13 @@ class CBuilder(QtWidgets.QMainWindow):
         self.mythread.window.layout.addWidget(q_label)
 
         self.wpanel.set_func_object_clicked(self.onItemClicked)
-
-        self.wmethods.set_wfunction(self.wfunction)
+        self.wmethods.set_func_object_clicked(self.form_wstackedwidget)
+        
+        self.wpanel.set_wfunction(self.wfunction)
+        self.wpanel.set_thread(self.mythread)
         self.wmethods.set_thread(self.mythread)
 
-        self.wdockarea.set_thread(self.mythread)
-        self.wbaseobj.set_thread(self.mythread)
+        self.wstackedwidget.set_thread(self.mythread)
 
         self.f_dir_data = f_dir_data
         self.f_file = None
@@ -200,13 +219,29 @@ class CBuilder(QtWidgets.QMainWindow):
             self.setWindowTitle(f'CrysPy editor: {self.f_file:}')
             self.load_globaln()
         else:
-            # self.wpanel.object = GlobalN.make_container((), (), "global")
             self.wpanel.object = RhoChi()
             self.form_wpanel()
             self.form_wstackedwidget(self.wpanel.object)
 
         self.show()
 
+    def get_dsetup(self):
+        """Get CrysPy editor parameters from files."""
+        f_setup = os.path.join(os.path.dirname(__file__), 'setup.json')
+        if os.path.isfile(f_setup):
+            d_setup = json.load(open(f_setup, 'r'))
+        else:
+            d_setup = {}
+        return d_setup
+
+    def save_dsetup(self, dsetup):
+        """Save CrysPy editor parameters to file."""
+        f_setup = os.path.join(os.path.dirname(__file__), 'setup.json')
+        s_cont = json.dumps(dsetup, sort_keys=True, indent=4)
+        with open(f_setup, "w") as fid:
+            fid.write(s_cont)
+
+
     def def_actions(self):
         """Define actions."""
         f_dir_prog = self.f_dir_prog
@@ -250,13 +285,14 @@ class CBuilder(QtWidgets.QMainWindow):
 
         fileMenu.addAction(save_as_action)
         self.toolbar.addAction(save_as_action)
+        
+        fileMenu.addSeparator()
 
-        # refresh_wind = QtWidgets.QAction(
-        #     QtGui.QIcon(os.path.join(f_dir_prog_icon, 'refresh.png')),
-        #     'Refresh', self)
-        # refresh_wind.setStatusTip('Refresh')
-        # refresh_wind.triggered.connect(self.form_wpanel)
-        # self.toolbar.addAction(refresh_wind)
+        exit_action = QtWidgets.QAction('&Exit', self)
+        exit_action.setShortcut('Ctrl+Q')
+        exit_action.setStatusTip('Exit application')
+        exit_action.triggered.connect(QtWidgets.qApp.quit)
+        fileMenu.addAction(exit_action)
 
         open_folder = QtWidgets.QAction(
             QtGui.QIcon(os.path.join(f_dir_prog_icon, 'open_folder.png')),
@@ -265,13 +301,6 @@ class CBuilder(QtWidgets.QMainWindow):
         open_folder.triggered.connect(self.open_folder)
         self.toolbar.addAction(open_folder)
 
-        # flip_action = QtWidgets.QAction(
-        #     QtGui.QIcon(os.path.join(f_dir_prog_icon, 'flip.png')),
-        #     'Flip representation', self)
-        # flip_action.setStatusTip('Flip presentation')
-        # flip_action.triggered.connect(self.flip_presenation)
-        # self.toolbar.addAction(flip_action)
-
         fileMenu = menubar.addMenu('&Object manipulations')
         manip_action = QtWidgets.QAction('Switch to global', fileMenu)
         manip_action.object = GlobalN
@@ -284,23 +313,38 @@ class CBuilder(QtWidgets.QMainWindow):
             manip_action.triggered.connect(self.transfer)
             fileMenu.addAction(manip_action)
 
-        about = menubar.addAction("About")
+        options_menu = menubar.addMenu('&Options')
+        action_packages = options_menu.addAction("Packages")
+        action_packages.triggered.connect(self.packages)
+
+        manual_site = options_menu.addAction("Manual (site)")
+        manual_site.triggered.connect(lambda x: os.startfile(r"https://ikibalin.github.io/cryspy/"))
+
+        about = options_menu.addAction("About")
         about.triggered.connect(self.about)
 
         self.toolbar_actions = self.addToolBar("Actions")
         self.statusBar()
 
+    def packages(self):
+        obj = WPackages(self)
+        obj.show()
+
     def about(self):
-        val = QtWidgets.QMessageBox.information(
+        s_address = r"https://github.com/ikibalin/cryspy"
+        QtWidgets.QMessageBox.information(
             self, "About CrysPy",
             f"Versions:\n\
     CrysPy Editor - {cryspy_editor_version:} \n\
     CrysPy library - {cryspy_version:}\n\n\
-To upgrade:\n\
-    python -m pip install --upgrade cryspy cryspy_editor --user\n\n\
-GitHub Page: \n\
-    https://github.com/ikibalin/cryspy")
+To upgrade see the site: \n\
+(address is copied to the clipboard) \n\n\
+{s_address:}")
 
+        cb = QtWidgets.QApplication.clipboard()
+        cb.clear(mode=cb.Clipboard)
+        cb.setText(s_address, mode=cb.Clipboard)
+    
     def transfer(self):
         """Transfer GlobalN object to any one."""
         sender = self.sender()
@@ -316,11 +360,6 @@ GitHub Page: \n\
         self.form_wpanel()
         self.form_wstackedwidget(self.wpanel.object)
 
-    def flip_presenation(self) -> NoReturn:
-        """Flip presentation."""
-        ind = 1-self.wstackedwidget.currentIndex()
-        self.wstackedwidget.setCurrentIndex(ind)
-
     def init_widget(self) -> NoReturn:
         """Initilize widget."""
         self.location_on_the_screen()
@@ -349,30 +388,80 @@ GitHub Page: \n\
         wsplitter_centr.setOrientation(QtCore.Qt.Vertical)
 
         # 1.
-        self.wstackedwidget = QtWidgets.QStackedWidget(wsplitter_centr)
-        self.wbaseobj = WBaseObj(self.wstackedwidget)
-        self.wdockarea = WDockArea(self.wstackedwidget)
-        self.wstackedwidget.addWidget(self.wbaseobj)
-        self.wstackedwidget.addWidget(self.wdockarea)
+        self.wstackedwidget = WBaseObj(wsplitter_centr)
         wsplitter_centr.addWidget(self.wstackedwidget)
 
         # 2.
+        # self.wnotes = WNotes(wsplitter_centr)
+        # self.wnotes.set_function_open(self.open_notes)
+        # self.wnotes.set_function_save_as(self.save_notes_as)
+        # self.wnotes.set_function_close_notes(self.close_notes)
+        # self.wnotes.set_function_notes_to_html(self.notes_to_html)
+        
+        # if ("file_notes" in self.dsetup.keys()):
+        #     self.wnotes.set_file_name(self.dsetup["file_notes"])
+        #     self.wnotes.open_notes()
+        # else:
+        #     self.wnotes.setText(S_NOTES)
+        #     self.wnotes.save()
+        # wsplitter_centr.addWidget(self.wnotes)
+
         self.wconsole = WConsole(wsplitter_centr)
         wsplitter_centr.addWidget(self.wconsole)
 
         wsplitter_centr.setSizes([width_v_1, width_v_2])
         wsplitter.addWidget(wsplitter_centr)
 
-        # Panel from right site
-        self.wmethods = WMethods(wsplitter)
+        # Panel from right site  
+        # self.wmethods = WMethods(wsplitter)
+        self.wmethods = WVariables(wsplitter)
         wsplitter.addWidget(self.wmethods)
 
-        wsplitter.setSizes([width_m_1, width_m_2, 0])
+        wsplitter.setSizes([width_m_1, width_m_2, width_m_1])
 
         layout_main.addWidget(wsplitter)
         widget_main.setLayout(layout_main)
         self.setCentralWidget(widget_main)
 
+    def notes_to_html(self, s_html: str):
+        self.wstackedwidget.w_notes.setHtml(s_html)
+
+    def open_notes(self):
+        if self.wnotes.file_name is not None:
+            f_dir = os.path.basename(self.wnotes.file_name)
+        else:
+            f_dir = self.f_dir_data
+
+        f_name, okPressed = QtWidgets.QFileDialog.getOpenFileName(
+            self, 'Select a cif file:', f_dir, "All files (*.*)")
+        if not (okPressed):
+            return None
+        if os.path.isfile(f_name):
+            self.wnotes.set_file_name(f_name)
+            self.wnotes.open_notes()
+
+        self.dsetup["file_notes"] = f_name
+        self.save_dsetup(self.dsetup)
+
+    def close_notes(self):
+        if "file_notes" in self.dsetup.keys():
+            del self.dsetup["file_notes"]
+            self.save_dsetup(self.dsetup)
+
+    def save_notes_as(self):
+        if self.wnotes.file_name is not None:
+            f_dir = os.path.basename(self.wnotes.file_name)
+        else:
+            f_dir = self.f_dir_data
+        f_name, okPressed = QtWidgets.QFileDialog.getSaveFileName(
+            self, 'Select a file:', f_dir, "All files (*.*)")
+        if not (okPressed):
+            return None
+        self.wnotes.set_file_name(f_name)
+        self.wnotes.save()
+        self.dsetup["file_notes"] = f_name
+        self.save_dsetup(self.dsetup)
+
     def location_on_the_screen(self) -> NoReturn:
         """Location on the screen."""
         screen = QtWidgets.QDesktopWidget().screenGeometry()
@@ -393,6 +482,7 @@ GitHub Page: \n\
         m_box.exec()
 
         # self.load_globaln()
+
     def load_globaln(self) -> NoReturn:
         """Read globaln from file."""
         try:
@@ -404,7 +494,8 @@ GitHub Page: \n\
         self.wpanel.object = obj
         self.form_wpanel()
         self.form_wstackedwidget(obj)
-        self.wmethods.get_methods(obj)
+        # self.wmethods.get_methods(obj)
+        self.wmethods.set_object(obj)
 
         w_item = self.wpanel.topLevelItem(0)
         self.wpanel.setCurrentItem(w_item)
@@ -415,7 +506,8 @@ GitHub Page: \n\
         self.wpanel.object = obj
         self.form_wpanel()
         self.form_wstackedwidget(obj)
-        self.wmethods.get_methods(obj)
+        # self.wmethods.get_methods(obj)
+        self.wmethods.set_object(obj)
 
     def save(self) -> NoReturn:
         """Save."""
@@ -467,7 +559,6 @@ GitHub Page: \n\
         with open(f_setup, "w") as fid:
             fid.write(s_cont)
 
-    # @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem, int)
     @QtCore.pyqtSlot(QtWidgets.QTreeWidgetItem, QtWidgets.QTreeWidgetItem)
     def onItemClicked(self, it, col):
         """On item clicked."""
@@ -510,8 +601,6 @@ GitHub Page: \n\
         self.wconsole.setText("\n".join(ls_out))
 
         self.form_wpanel()
-        # self.form_wstackedwidget(self.wpanel.object)
-        # self.wmethods.get_methods(self.wpanel.object)
 
         previous_item = self.index_curent_item
 
@@ -530,9 +619,7 @@ GitHub Page: \n\
             w_item = self.wpanel.topLevelItem(0)
         self.wpanel.setCurrentItem(w_item)
         self.form_wstackedwidget(w_item.object)
-        self.wmethods.get_methods(w_item.object)
-
-        # self.calc_is_finished()
+        self.wmethods.get_variables()
 
     def form_wpanel(self):
         """Form wpanel."""
@@ -540,18 +627,14 @@ GitHub Page: \n\
 
     def form_wstackedwidget(self, obj):
         """From stackwidget."""
-        self.wbaseobj.set_object(obj)
-        w_actions = self.wdockarea.set_object(obj)
+        self.wstackedwidget.set_object(obj)
+        tool_buttons = define_tool_buttons(obj, self.mythread)
 
         self.toolbar_actions.clear()
-        for w_action in w_actions:
-            self.toolbar_actions.addWidget(w_action)
+        for tool_button in tool_buttons:
+            self.toolbar_actions.addWidget(tool_button)
 
-        if self.wdockarea.count() == 0:
-            self.wstackedwidget.setCurrentIndex(0)
-        else:
-            self.wstackedwidget.setCurrentIndex(1)
-        self.wmethods.get_methods(obj)
+        self.wmethods.get_variables()
 
 
 def main_w(l_arg=[]):
