@@ -1,11 +1,12 @@
-"""WPanel."""
 from typing import Union, Callable, Any, NoReturn
+from types import FunctionType
 import copy
 from PyQt5 import QtCore, QtGui, QtWidgets
 
 from cryspy import ItemN, LoopN, DataN, GlobalN, str_to_items, \
     str_to_globaln
 
+
 from cryspy import L_GLOBAL_CLASS, L_DATA_CLASS, L_ITEM_CLASS, L_LOOP_CLASS
 L_GLOBAL_CLS = L_GLOBAL_CLASS
 L_DATA_CLS = L_DATA_CLASS
@@ -28,8 +29,6 @@ class WPanel(QtWidgets.QTreeWidget):
         self.object = None
 
         self.customContextMenuRequested.connect(self.open_menu)
-        # for key, item in kwargs.keys():
-        #     if key == ""
 
     def set_object_global(self) \
             -> NoReturn:
@@ -100,14 +99,6 @@ class WPanel(QtWidgets.QTreeWidget):
             # l_ind = find_tree_item_position(self, w_item)
             obj = w_item.object
             menu = QtWidgets.QMenu(self)
-            save_as = QtWidgets.QAction('Save to file ...', menu)
-            save_as.triggered.connect(lambda x: self.save_object(obj))
-            menu.addAction(save_as)
-
-            if isinstance(obj, (GlobalN, DataN)):
-                add_items = QtWidgets.QAction('Add items by cif ...', menu)
-                add_items.triggered.connect(lambda: self.add_items(w_item))
-                menu.addAction(add_items)
 
             if ((type(obj) is GlobalN) | (type(obj) is DataN)):
                 menu_item = menu.addMenu("Add")
@@ -121,8 +112,7 @@ class WPanel(QtWidgets.QTreeWidget):
                         prefix = cls_item.PREFIX
                     else:
                         prefix = cls_item.ITEM_CLASS.PREFIX
-                    add_item = QtWidgets.QAction(f'{prefix :}',
-                                                 menu_item)
+                    add_item = QtWidgets.QAction(f'{prefix :}', menu_item)
                     add_item.cls_item = cls_item
                     add_item.triggered.connect(lambda: self.add_item(w_item))
                     menu_item.addAction(add_item)
@@ -131,13 +121,12 @@ class WPanel(QtWidgets.QTreeWidget):
                 menu_item = menu.addMenu("Add")
                 for cls_item in obj.CLASSES_MANDATORY:
                     if ((cls_item is not DataN) & (cls_item is not LoopN) &
-                            (cls_item is not ItemN)):
+                        (cls_item is not ItemN)):
                         if "PREFIX" in cls_item.__dict__.keys():
                             prefix = cls_item.PREFIX
                         else:
                             prefix = cls_item.ITEM_CLASS.PREFIX
-                        add_item = QtWidgets.QAction(f'{prefix:}',
-                                                     menu_item)
+                        add_item = QtWidgets.QAction(f'{prefix:}', menu_item)
                         add_item.cls_item = cls_item
                         add_item.triggered.connect(
                             lambda: self.add_item(w_item))
@@ -150,8 +139,7 @@ class WPanel(QtWidgets.QTreeWidget):
                             prefix = cls_item.PREFIX
                         else:
                             prefix = cls_item.ITEM_CLASS.PREFIX
-                        add_item = QtWidgets.QAction(f'{prefix :}',
-                                                     menu_item)
+                        add_item = QtWidgets.QAction(f'{prefix :}', menu_item)
                         add_item.cls_item = cls_item
                         add_item.triggered.connect(
                             lambda: self.add_item(w_item))
@@ -198,8 +186,44 @@ class WPanel(QtWidgets.QTreeWidget):
                 qaction.triggered.connect(lambda: self.duplicate_data(w_item))
                 menu.addAction(qaction)
 
+            obj = w_item.object
+            l_method = [_1 for _1, _2 in type(obj).__dict__.items()
+                        if ((type(_2) == FunctionType) &
+                            (not(_1.startswith("_"))))]
+            if len(l_method) != 0:
+                menu_method = menu.addMenu("Methods")
+                for method in l_method:
+                    func = getattr(obj, method)
+                    l_param = [_ for _ in func.__code__.co_varnames[
+                        :func.__code__.co_argcount] if _ != "self"]
+
+                    s_par = ""
+                    if len(l_param) > 0:
+                        s_par = ", ".join(l_param)
+                    s_val = f"{method:}({s_par:})"
+                    qaction = QtWidgets.QAction(s_val, menu_method)
+                    qaction.object = obj
+                    qaction.triggered.connect(lambda x: self.do_func())
+                    menu_method.addAction(qaction)
+            
             menu.exec_(self.viewport().mapToGlobal(position))
 
+    def do_func(self):
+        sender = self.sender()
+        obj = sender.object
+        name = sender.text()
+        func_name = name.split("(")[0]
+        func = getattr(obj, func_name)
+        self.wfunction.set_function(func, self.mythread)
+        
+    def set_thread(self, thread):
+        """Set text."""
+        self.mythread = thread
+
+    def set_wfunction(self, wfunction):
+        """Set text."""
+        self.wfunction = wfunction
+
     def duplicate_data(self, widget: QtWidgets.QTreeWidgetItem):
         obj_data = widget.object
         widget_parent = widget.parent()
@@ -401,8 +425,6 @@ class MWAddItem(QtWidgets.QMainWindow):
         elif isinstance(obj, DataN):
             l_item = str_to_items(s_cont)
 
-        str_to_globaln
-
         cls_opt_obj = obj.CLASSES_OPTIONAL
         cls_obj = obj.CLASSES
         if ((obj_cls is DataN) | (obj_cls is GlobalN)):
